@@ -1,34 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Gui module
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -214,9 +217,6 @@ QT_BEGIN_NAMESPACE
     QShader, it indicates no shader code was found for the requested key.
  */
 
-static const int QSB_VERSION = 2;
-static const int QSB_VERSION_WITHOUT_BINDINGS = 1;
-
 /*!
     Constructs a new, empty (and thus invalid) QShader instance.
  */
@@ -307,7 +307,7 @@ void QShader::setDescription(const QShaderDescription &desc)
 /*!
     \return the list of available shader versions
  */
-QVector<QShaderKey> QShader::availableShaders() const
+QList<QShaderKey> QShader::availableShaders() const
 {
     return d->shaders.keys().toVector();
 }
@@ -368,10 +368,10 @@ QByteArray QShader::serialized() const
     if (!buf.open(QIODevice::WriteOnly))
         return QByteArray();
 
-    ds << QSB_VERSION;
+    ds << QShaderPrivate::QSB_VERSION;
     ds << int(d->stage);
-    ds << d->desc.toBinaryJson();
-    ds << d->shaders.count();
+    d->desc.serialize(&ds);
+    ds << int(d->shaders.count());
     for (auto it = d->shaders.cbegin(), itEnd = d->shaders.cend(); it != itEnd; ++it) {
         const QShaderKey &k(it.key());
         writeShaderKey(&ds, k);
@@ -379,12 +379,12 @@ QByteArray QShader::serialized() const
         ds << shader.shader();
         ds << shader.entryPoint();
     }
-    ds << d->bindings.count();
+    ds << int(d->bindings.count());
     for (auto it = d->bindings.cbegin(), itEnd = d->bindings.cend(); it != itEnd; ++it) {
         const QShaderKey &k(it.key());
         writeShaderKey(&ds, k);
         const NativeResourceBindingMap &map(it.value());
-        ds << map.count();
+        ds << int(map.count());
         for (auto mapIt = map.cbegin(), mapItEnd = map.cend(); mapIt != mapItEnd; ++mapIt) {
             ds << mapIt.key();
             ds << mapIt.value().first;
@@ -429,17 +429,28 @@ QShader QShader::fromSerialized(const QByteArray &data)
     Q_ASSERT(d->ref.loadRelaxed() == 1); // must be detached
     int intVal;
     ds >> intVal;
-    const int qsbVersion = intVal;
-    if (qsbVersion != QSB_VERSION && qsbVersion != QSB_VERSION_WITHOUT_BINDINGS) {
-        qWarning("Attempted to deserialize QShader with unknown version %d.", qsbVersion);
+    d->qsbVersion = intVal;
+    if (d->qsbVersion != QShaderPrivate::QSB_VERSION
+            && d->qsbVersion != QShaderPrivate::QSB_VERSION_WITHOUT_VAR_ARRAYDIMS
+            && d->qsbVersion != QShaderPrivate::QSB_VERSION_WITH_CBOR
+            && d->qsbVersion != QShaderPrivate::QSB_VERSION_WITH_BINARY_JSON
+            && d->qsbVersion != QShaderPrivate::QSB_VERSION_WITHOUT_BINDINGS)
+    {
+        qWarning("Attempted to deserialize QShader with unknown version %d.", d->qsbVersion);
         return QShader();
     }
 
     ds >> intVal;
     d->stage = Stage(intVal);
-    QByteArray descBin;
-    ds >> descBin;
-    d->desc = QShaderDescription::fromBinaryJson(descBin);
+    if (d->qsbVersion > QShaderPrivate::QSB_VERSION_WITH_CBOR) {
+        d->desc = QShaderDescription::deserialize(&ds, d->qsbVersion);
+    } else if (d->qsbVersion > QShaderPrivate::QSB_VERSION_WITH_BINARY_JSON) {
+        qWarning("Can no longer load QShaderDescription from CBOR.");
+        d->desc = QShaderDescription();
+    } else {
+        qWarning("Can no longer load QShaderDescription from binary JSON.");
+        d->desc = QShaderDescription();
+    }
     int count;
     ds >> count;
     for (int i = 0; i < count; ++i) {
@@ -454,7 +465,7 @@ QShader QShader::fromSerialized(const QByteArray &data)
         d->shaders[k] = shader;
     }
 
-    if (qsbVersion != QSB_VERSION_WITHOUT_BINDINGS) {
+    if (d->qsbVersion > QShaderPrivate::QSB_VERSION_WITHOUT_BINDINGS) {
         ds >> count;
         for (int i = 0; i < count; ++i) {
             QShaderKey k;
@@ -504,7 +515,7 @@ QShaderKey::QShaderKey(QShader::Source s,
 
     \relates QShader
  */
-bool operator==(const QShader &lhs, const QShader &rhs) Q_DECL_NOTHROW
+bool operator==(const QShader &lhs, const QShader &rhs) noexcept
 {
     return lhs.d->stage == rhs.d->stage
             && lhs.d->shaders == rhs.d->shaders;
@@ -512,6 +523,7 @@ bool operator==(const QShader &lhs, const QShader &rhs) Q_DECL_NOTHROW
 }
 
 /*!
+    \internal
     \fn bool operator!=(const QShader &lhs, const QShader &rhs)
 
     Returns \c false if the values in the two QShader objects \a a and \a b
@@ -525,12 +537,14 @@ bool operator==(const QShader &lhs, const QShader &rhs) Q_DECL_NOTHROW
 
     \relates QShader
  */
-uint qHash(const QShader &s, uint seed) Q_DECL_NOTHROW
+size_t qHash(const QShader &s, size_t seed) noexcept
 {
-    uint h = s.stage();
-    for (auto it = s.d->shaders.constBegin(), itEnd = s.d->shaders.constEnd(); it != itEnd; ++it)
-        h += qHash(it.key(), seed) + qHash(it.value().shader(), seed);
-    return h;
+    QtPrivate::QHashCombine hash;
+    seed = hash(seed, s.stage());
+    seed = qHashRange(s.d->shaders.keyValueBegin(),
+                      s.d->shaders.keyValueEnd(),
+                      seed);
+    return seed;
 }
 
 /*!
@@ -539,12 +553,13 @@ uint qHash(const QShader &s, uint seed) Q_DECL_NOTHROW
 
     \relates QShaderVersion
  */
-bool operator==(const QShaderVersion &lhs, const QShaderVersion &rhs) Q_DECL_NOTHROW
+bool operator==(const QShaderVersion &lhs, const QShaderVersion &rhs) noexcept
 {
     return lhs.version() == rhs.version() && lhs.flags() == rhs.flags();
 }
 
 /*!
+    \internal
     \fn bool operator!=(const QShaderVersion &lhs, const QShaderVersion &rhs)
 
     Returns \c false if the values in the two QShaderVersion objects \a a
@@ -558,13 +573,14 @@ bool operator==(const QShaderVersion &lhs, const QShaderVersion &rhs) Q_DECL_NOT
 
     \relates QShaderKey
  */
-bool operator==(const QShaderKey &lhs, const QShaderKey &rhs) Q_DECL_NOTHROW
+bool operator==(const QShaderKey &lhs, const QShaderKey &rhs) noexcept
 {
     return lhs.source() == rhs.source() && lhs.sourceVersion() == rhs.sourceVersion()
             && lhs.sourceVariant() == rhs.sourceVariant();
 }
 
 /*!
+    \internal
     \fn bool operator!=(const QShaderKey &lhs, const QShaderKey &rhs)
 
     Returns \c false if the values in the two QShaderKey objects \a a
@@ -578,9 +594,13 @@ bool operator==(const QShaderKey &lhs, const QShaderKey &rhs) Q_DECL_NOTHROW
 
     \relates QShaderKey
  */
-uint qHash(const QShaderKey &k, uint seed) Q_DECL_NOTHROW
+size_t qHash(const QShaderKey &k, size_t seed) noexcept
 {
-    return seed + 10 * k.source() + k.sourceVersion().version() + k.sourceVersion().flags() + k.sourceVariant();
+    return qHashMulti(seed,
+                      k.source(),
+                      k.sourceVersion().version(),
+                      k.sourceVersion().flags(),
+                      k.sourceVariant());
 }
 
 /*!
@@ -588,12 +608,13 @@ uint qHash(const QShaderKey &k, uint seed) Q_DECL_NOTHROW
 
     \relates QShaderCode
  */
-bool operator==(const QShaderCode &lhs, const QShaderCode &rhs) Q_DECL_NOTHROW
+bool operator==(const QShaderCode &lhs, const QShaderCode &rhs) noexcept
 {
     return lhs.shader() == rhs.shader() && lhs.entryPoint() == rhs.entryPoint();
 }
 
 /*!
+    \internal
     \fn bool operator!=(const QShaderCode &lhs, const QShaderCode &rhs)
 
     Returns \c false if the values in the two QShaderCode objects \a a
@@ -602,6 +623,16 @@ bool operator==(const QShaderCode &lhs, const QShaderCode &rhs) Q_DECL_NOTHROW
     \relates QShaderCode
  */
 
+/*!
+    Returns the hash value for \a k, using \a seed to seed the calculation.
+
+    \relates QShaderCode
+ */
+size_t qHash(const QShaderCode &k, size_t seed) noexcept
+{
+    return qHash(k.shader(), seed);
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug dbg, const QShader &bs)
 {
@@ -657,6 +688,13 @@ QDebug operator<<(QDebug dbg, const QShaderVersion &v)
     pair, because combined image samplers may map to two native resources (a
     texture and a sampler) in some shading languages. In that case the second
     value refers to the sampler.
+
+    \note The native binding may be -1, in case there is no active binding for
+    the resource in the shader. (for example, there is a uniform block
+    declared, but it is not used in the shader code) The map is always
+    complete, meaning there is an entry for all declared uniform blocks,
+    storage blocks, image objects, and combined samplers, but the value will be
+    -1 for those that are not actually referenced in the shader functions.
 */
 
 /*!
