@@ -425,7 +425,7 @@ bool QRfbPointerEvent::read(QTcpSocket *s)
     if (buttonMask & 1)
         buttons |= Qt::LeftButton;
     if (buttonMask & 2)
-        buttons |= Qt::MidButton;
+        buttons |= Qt::MiddleButton;
     if (buttonMask & 4)
         buttons |= Qt::RightButton;
 
@@ -503,7 +503,7 @@ void QRfbRawEncoder::write()
         const quint32 encoding = htonl(0); // raw encoding
         socket->write((char *)&encoding, sizeof(encoding));
 
-        int linestep = screenImage.bytesPerLine();
+        qsizetype linestep = screenImage.bytesPerLine();
         const uchar *screendata = screenImage.scanLine(rect.y)
                                   + rect.x * screenImage.depth() / 8;
 
@@ -515,8 +515,9 @@ void QRfbRawEncoder::write()
             // convert pixels
             char *b = buffer.data();
             const int bstep = rect.w * bytesPerPixel;
+            const int depth = screenImage.depth();
             for (int i = 0; i < rect.h; ++i) {
-                client->convertPixels(b, (const char*)screendata, rect.w);
+                client->convertPixels(b, (const char*)screendata, rect.w, depth);
                 screendata += linestep;
                 b += bstep;
             }
@@ -538,7 +539,7 @@ QVncClientCursor::QVncClientCursor()
 {
     QWindow *w = QGuiApplication::focusWindow();
     QCursor c = w ? w->cursor() : QCursor(Qt::ArrowCursor);
-    changeCursor(&c, 0);
+    changeCursor(&c, nullptr);
 }
 
 QVncClientCursor::~QVncClientCursor()
@@ -569,9 +570,10 @@ void QVncClientCursor::write(QVncClient *client) const
     Q_ASSERT(cursor.hasAlphaChannel());
     const QImage img = cursor.convertToFormat(client->server()->screen()->format());
     const int n = client->clientBytesPerPixel() * img.width();
+    const int depth = img.depth();
     char *buffer = new char[n];
     for (int i = 0; i < img.height(); ++i) {
-        client->convertPixels(buffer, (const char*)img.scanLine(i), img.width());
+        client->convertPixels(buffer, (const char*)img.scanLine(i), img.width(), depth);
         socket->write(buffer, n);
     }
     delete[] buffer;
@@ -596,19 +598,22 @@ void QVncClientCursor::changeCursor(QCursor *widgetCursor, QWindow *window)
         cursor = widgetCursor->pixmap().toImage();
     } else {
         // system cursor
-        QPlatformCursorImage platformImage(0, 0, 0, 0, 0, 0);
+        QPlatformCursorImage platformImage(nullptr, nullptr, 0, 0, 0, 0);
         platformImage.set(shape);
         cursor = *platformImage.image();
         hotspot = platformImage.hotspot();
     }
-    for (auto client : clients)
+    for (auto client : qAsConst(clients))
         client->setDirtyCursor();
 }
 
 void QVncClientCursor::addClient(QVncClient *client)
 {
-    if (!clients.contains(client))
+    if (!clients.contains(client)) {
         clients.append(client);
+        // Force a cursor update when the client connects.
+        client->setDirtyCursor();
+    }
 }
 
 uint QVncClientCursor::removeClient(QVncClient *client)
@@ -639,16 +644,14 @@ void QVncServer::init()
 
 QVncServer::~QVncServer()
 {
-    for (auto client : clients) {
-        delete client;
-    }
+    qDeleteAll(clients);
 }
 
 void QVncServer::setDirty()
 {
-    for (auto client : clients) {
+    for (auto client : qAsConst(clients))
         client->setDirty(qvnc_screen->dirtyRegion);
-    }
+
     qvnc_screen->clearDirty();
 }
 
