@@ -67,10 +67,10 @@ enum {
 };
 #undef CursorShape
 
-static PtrXcursorLibraryLoadCursor ptrXcursorLibraryLoadCursor = 0;
-static PtrXcursorLibraryGetTheme ptrXcursorLibraryGetTheme = 0;
-static PtrXcursorLibrarySetTheme ptrXcursorLibrarySetTheme = 0;
-static PtrXcursorLibraryGetDefaultSize ptrXcursorLibraryGetDefaultSize = 0;
+static PtrXcursorLibraryLoadCursor ptrXcursorLibraryLoadCursor = nullptr;
+static PtrXcursorLibraryGetTheme ptrXcursorLibraryGetTheme = nullptr;
+static PtrXcursorLibrarySetTheme ptrXcursorLibrarySetTheme = nullptr;
+static PtrXcursorLibraryGetDefaultSize ptrXcursorLibraryGetDefaultSize = nullptr;
 #endif
 
 static xcb_font_t cursorFont = 0;
@@ -118,7 +118,7 @@ static const uint8_t mcur_fdiag_bits[] = {
 static const uint8_t *cursor_bits16[] = {
     cur_ver_bits, mcur_ver_bits, cur_hor_bits, mcur_hor_bits,
     cur_bdiag_bits, mcur_bdiag_bits, cur_fdiag_bits, mcur_fdiag_bits,
-    0, 0, cur_blank_bits, cur_blank_bits };
+    nullptr, nullptr, cur_blank_bits, cur_blank_bits };
 
 static const uint8_t vsplit_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -219,7 +219,7 @@ static const uint8_t busym_bits[] = {
 
 static const uint8_t * const cursor_bits32[] = {
     vsplit_bits, vsplitm_bits, hsplit_bits, hsplitm_bits,
-    0, 0, 0, 0, whatsthis_bits, whatsthism_bits, busy_bits, busym_bits
+    nullptr, nullptr, nullptr, nullptr, whatsthis_bits, whatsthism_bits, busy_bits, busym_bits
 };
 
 static const uint8_t forbidden_bits[] = {
@@ -255,6 +255,7 @@ static const uint8_t * const cursor_bits20[] = {
     forbidden_bits, forbiddenm_bits
 };
 
+// ### FIXME This mapping is incomplete - QTBUG-71423
 static const std::vector<const char *> cursorNames[] = {
     { "left_ptr", "default", "top_left_arrow", "left_arrow" },
     { "up_arrow" },
@@ -273,7 +274,7 @@ static const std::vector<const char *> cursorNames[] = {
     { "forbidden", "not-allowed", "crossed_circle", "circle", "03b6e0fcb3499374a867c041f52298f0" },
     { "whats_this", "help", "question_arrow", "5c6cd98b3f3ebcb1f9c7f1c204630408", "d9ce0ab605698f320427677b458ad60b" },
     { "left_ptr_watch", "half-busy", "progress", "00000000000000020006000e7e9ffc3f", "08e8e1c95fe2fc01f976f1e063a24ccd" },
-    { "openhand", "fleur", "5aca4d189052212118709018842178c0", "9d800788f1b08800ae810202380a0822" },
+    { "openhand", "grab", "fleur", "5aca4d189052212118709018842178c0", "9d800788f1b08800ae810202380a0822" },
     { "closedhand", "grabbing", "208530c400c041818281048008011002" },
     { "dnd-copy", "copy" },
     { "dnd-move", "move" },
@@ -288,10 +289,10 @@ QXcbCursorCacheKey::QXcbCursorCacheKey(const QCursor &c)
         if (pixmapCacheKey) {
             bitmapCacheKey = pixmapCacheKey;
         } else {
-            Q_ASSERT(c.bitmap());
-            Q_ASSERT(c.mask());
-            bitmapCacheKey = c.bitmap()->cacheKey();
-            maskCacheKey = c.mask()->cacheKey();
+            Q_ASSERT(!c.bitmap(Qt::ReturnByValue).isNull());
+            Q_ASSERT(!c.mask(Qt::ReturnByValue).isNull());
+            bitmapCacheKey = c.bitmap(Qt::ReturnByValue).cacheKey();
+            maskCacheKey = c.mask(Qt::ReturnByValue).cacheKey();
         }
     }
 }
@@ -451,19 +452,19 @@ xcb_cursor_t QXcbCursor::createNonStandardCursor(int cshape)
 
     if (cshape == Qt::BlankCursor) {
         xcb_pixmap_t cp = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(), cur_blank_bits, 16, 16,
-                                                             1, 0, 0, 0);
+                                                             1, 0, 0, nullptr);
         xcb_pixmap_t mp = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(), cur_blank_bits, 16, 16,
-                                                             1, 0, 0, 0);
+                                                             1, 0, 0, nullptr);
         cursor = xcb_generate_id(conn);
         xcb_create_cursor(conn, cursor, cp, mp, 0, 0, 0, 0xFFFF, 0xFFFF, 0xFFFF, 8, 8);
     } else if (cshape >= Qt::SizeVerCursor && cshape < Qt::SizeAllCursor) {
         int i = (cshape - Qt::SizeVerCursor) * 2;
         xcb_pixmap_t pm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                              const_cast<uint8_t*>(cursor_bits16[i]),
-                                                             16, 16, 1, 0, 0, 0);
+                                                             16, 16, 1, 0, 0, nullptr);
         xcb_pixmap_t pmm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                               const_cast<uint8_t*>(cursor_bits16[i + 1]),
-                                                              16, 16, 1, 0, 0, 0);
+                                                              16, 16, 1, 0, 0, nullptr);
         cursor = xcb_generate_id(conn);
         xcb_create_cursor(conn, cursor, pm, pmm, 0, 0, 0, 0xFFFF, 0xFFFF, 0xFFFF, 8, 8);
     } else if ((cshape >= Qt::SplitVCursor && cshape <= Qt::SplitHCursor)
@@ -471,10 +472,10 @@ xcb_cursor_t QXcbCursor::createNonStandardCursor(int cshape)
         int i = (cshape - Qt::SplitVCursor) * 2;
         xcb_pixmap_t pm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                              const_cast<uint8_t*>(cursor_bits32[i]),
-                                                             32, 32, 1, 0, 0, 0);
+                                                             32, 32, 1, 0, 0, nullptr);
         xcb_pixmap_t pmm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                               const_cast<uint8_t*>(cursor_bits32[i + 1]),
-                                                              32, 32, 1, 0, 0, 0);
+                                                              32, 32, 1, 0, 0, nullptr);
         int hs = (cshape == Qt::PointingHandCursor || cshape == Qt::WhatsThisCursor
                   || cshape == Qt::BusyCursor) ? 0 : 16;
         cursor = xcb_generate_id(conn);
@@ -483,20 +484,20 @@ xcb_cursor_t QXcbCursor::createNonStandardCursor(int cshape)
         int i = (cshape - Qt::ForbiddenCursor) * 2;
         xcb_pixmap_t pm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                              const_cast<uint8_t*>(cursor_bits20[i]),
-                                                             20, 20, 1, 0, 0, 0);
+                                                             20, 20, 1, 0, 0, nullptr);
         xcb_pixmap_t pmm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                               const_cast<uint8_t*>(cursor_bits20[i + 1]),
-                                                              20, 20, 1, 0, 0, 0);
+                                                              20, 20, 1, 0, 0, nullptr);
         cursor = xcb_generate_id(conn);
         xcb_create_cursor(conn, cursor, pm, pmm, 0, 0, 0, 0xFFFF, 0xFFFF, 0xFFFF, 10, 10);
     } else if (cshape == Qt::OpenHandCursor || cshape == Qt::ClosedHandCursor) {
         bool open = cshape == Qt::OpenHandCursor;
         xcb_pixmap_t pm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                              const_cast<uint8_t*>(open ? openhand_bits : closedhand_bits),
-                                                             16, 16, 1, 0, 0, 0);
+                                                             16, 16, 1, 0, 0, nullptr);
         xcb_pixmap_t pmm = xcb_create_pixmap_from_bitmap_data(conn, m_screen->root(),
                                                              const_cast<uint8_t*>(open ? openhandm_bits : closedhandm_bits),
-                                                             16, 16, 1, 0, 0, 0);
+                                                             16, 16, 1, 0, 0, nullptr);
         cursor = xcb_generate_id(conn);
         xcb_create_cursor(conn, cursor, pm, pmm, 0, 0, 0, 0xFFFF, 0xFFFF, 0xFFFF, 8, 8);
     } else if (cshape == Qt::DragCopyCursor || cshape == Qt::DragMoveCursor
@@ -606,18 +607,14 @@ xcb_cursor_t QXcbCursor::createBitmapCursor(QCursor *cursor)
     QPoint spot = cursor->hotSpot();
     xcb_cursor_t c = XCB_NONE;
     if (cursor->pixmap().depth() > 1) {
-#if QT_CONFIG(xcb_render)
         if (connection()->hasXRender(0, 5))
             c = qt_xcb_createCursorXRender(m_screen, cursor->pixmap().toImage(), spot);
         else
             qCWarning(lcQpaXcb, "xrender >= 0.5 required to create pixmap cursors");
-#else
-        qCWarning(lcQpaXcb, "This build of Qt does not support pixmap cursors");
-#endif
     } else {
         xcb_connection_t *conn = xcb_connection();
-        xcb_pixmap_t cp = qt_xcb_XPixmapFromBitmap(m_screen, cursor->bitmap()->toImage());
-        xcb_pixmap_t mp = qt_xcb_XPixmapFromBitmap(m_screen, cursor->mask()->toImage());
+        xcb_pixmap_t cp = qt_xcb_XPixmapFromBitmap(m_screen, cursor->bitmap(Qt::ReturnByValue).toImage());
+        xcb_pixmap_t mp = qt_xcb_XPixmapFromBitmap(m_screen, cursor->mask(Qt::ReturnByValue).toImage());
         c = xcb_generate_id(conn);
         xcb_create_cursor(conn, c, cp, mp, 0, 0, 0, 0xFFFF, 0xFFFF, 0xFFFF,
                           spot.x(), spot.y());
@@ -663,15 +660,16 @@ void QXcbCursor::queryPointer(QXcbConnection *c, QXcbVirtualDesktop **virtualDes
 QPoint QXcbCursor::pos() const
 {
     QPoint p;
-    queryPointer(connection(), 0, &p);
+    queryPointer(connection(), nullptr, &p);
     return p;
 }
 
 void QXcbCursor::setPos(const QPoint &pos)
 {
     QXcbVirtualDesktop *virtualDesktop = nullptr;
-    queryPointer(connection(), &virtualDesktop, 0);
-    xcb_warp_pointer(xcb_connection(), XCB_NONE, virtualDesktop->root(), 0, 0, 0, 0, pos.x(), pos.y());
+    queryPointer(connection(), &virtualDesktop, nullptr);
+    if (virtualDesktop)
+        xcb_warp_pointer(xcb_connection(), XCB_NONE, virtualDesktop->root(), 0, 0, 0, 0, pos.x(), pos.y());
     xcb_flush(xcb_connection());
 }
 
