@@ -46,8 +46,6 @@
 #include <qevent.h>
 #include <qapplication.h>
 #include <private/qcombobox_p.h>
-#include <QDesktopWidget>
-#include <private/qdesktopwidget_p.h>
 #include <qdebug.h>
 
 QT_BEGIN_NAMESPACE
@@ -136,8 +134,8 @@ static QFontDatabase::WritingSystem writingSystemFromLocale()
 
 static QFontDatabase::WritingSystem writingSystemForFont(const QFont &font, bool *hasLatin)
 {
-    QList<QFontDatabase::WritingSystem> writingSystems = QFontDatabase().writingSystems(font.family());
-//     qDebug() << font.family() << writingSystems;
+    QList<QFontDatabase::WritingSystem> writingSystems = QFontDatabase::writingSystems(font.families().first());
+//     qDebug() << font.families().first() << writingSystems;
 
     // this just confuses the algorithm below. Vietnamese is Latin with lots of special chars
     writingSystems.removeOne(QFontDatabase::Vietnamese);
@@ -189,10 +187,10 @@ public:
     // painting
     void paint(QPainter *painter,
                const QStyleOptionViewItem &option,
-               const QModelIndex &index) const Q_DECL_OVERRIDE;
+               const QModelIndex &index) const override;
 
     QSize sizeHint(const QStyleOptionViewItem &option,
-                   const QModelIndex &index) const Q_DECL_OVERRIDE;
+                   const QModelIndex &index) const override;
 
     const QIcon truetype;
     const QIcon bitmap;
@@ -215,7 +213,7 @@ void QFontFamilyDelegate::paint(QPainter *painter,
     QFont font(option.font);
     font.setPointSize(QFontInfo(font).pointSize() * 3 / 2);
     QFont font2 = font;
-    font2.setFamily(text);
+    font2.setFamilies(QStringList{text});
 
     bool hasLatin;
     QFontDatabase::WritingSystem system = writingSystemForFont(font2, &hasLatin);
@@ -233,12 +231,13 @@ void QFontFamilyDelegate::paint(QPainter *painter,
     }
 
     const QIcon *icon = &bitmap;
-    if (QFontDatabase().isSmoothlyScalable(text)) {
+    if (QFontDatabase::isSmoothlyScalable(text)) {
         icon = &truetype;
     }
-    QSize actualSize = icon->actualSize(r.size());
-
-    icon->paint(painter, r, Qt::AlignLeft|Qt::AlignVCenter);
+    const QSize actualSize = icon->actualSize(r.size());
+    const QRect iconRect = QStyle::alignedRect(option.direction, option.displayAlignment,
+                                               actualSize, r);
+    icon->paint(painter, iconRect, Qt::AlignLeft|Qt::AlignVCenter);
     if (option.direction == Qt::RightToLeft)
         r.setRight(r.right() - actualSize.width() - 4);
     else
@@ -247,6 +246,7 @@ void QFontFamilyDelegate::paint(QPainter *painter,
     QFont old = painter->font();
     painter->setFont(font);
 
+    const Qt::Alignment textAlign = QStyle::visualAlignment(option.direction, option.displayAlignment);
     // If the ascent of the font is larger than the height of the rect,
     // we will clip the text, so it's better to align the tight bounding rect in this case
     // This is specifically for fonts where the ascent is very large compared to
@@ -254,23 +254,25 @@ void QFontFamilyDelegate::paint(QPainter *painter,
     QFontMetricsF fontMetrics(font);
     if (fontMetrics.ascent() > r.height()) {
         QRectF tbr = fontMetrics.tightBoundingRect(text);
-        painter->drawText(r.x(), r.y() + (r.height() + tbr.height()) / 2.0, text);
+        QRect textRect(r);
+        textRect.setHeight(textRect.height() + (r.height() - tbr.height()));
+        painter->drawText(textRect, Qt::AlignBottom|Qt::TextSingleLine|textAlign, text);
     } else {
-        painter->drawText(r, Qt::AlignVCenter|Qt::AlignLeading|Qt::TextSingleLine, text);
+        painter->drawText(r, Qt::AlignVCenter|Qt::TextSingleLine|textAlign, text);
     }
 
     if (writingSystem != QFontDatabase::Any)
         system = writingSystem;
 
     if (system != QFontDatabase::Any) {
-        int w = painter->fontMetrics().width(text + QLatin1String("  "));
+        int w = painter->fontMetrics().horizontalAdvance(text + QLatin1String("  "));
         painter->setFont(font2);
-        QString sample = QFontDatabase().writingSystemSample(system);
+        QString sample = QFontDatabase::writingSystemSample(system);
         if (option.direction == Qt::RightToLeft)
             r.setRight(r.right() - w);
         else
             r.setLeft(r.left() + w);
-        painter->drawText(r, Qt::AlignVCenter|Qt::AlignLeading|Qt::TextSingleLine, sample);
+        painter->drawText(r, Qt::AlignVCenter|Qt::TextSingleLine|textAlign, sample);
     }
     painter->setFont(old);
 
@@ -284,10 +286,10 @@ QSize QFontFamilyDelegate::sizeHint(const QStyleOptionViewItem &option,
 {
     QString text = index.data(Qt::DisplayRole).toString();
     QFont font(option.font);
-//     font.setFamily(text);
+//     font.setFamilies(QStringList{text});
     font.setPointSize(QFontInfo(font).pointSize() * 3/2);
     QFontMetrics fontMetrics(font);
-    return QSize(fontMetrics.width(text), fontMetrics.height());
+    return QSize(fontMetrics.horizontalAdvance(text), fontMetrics.height());
 }
 
 
@@ -318,23 +320,22 @@ void QFontComboBoxPrivate::_q_updateModel()
     QFontFamilyDelegate *delegate = qobject_cast<QFontFamilyDelegate *>(q->view()->itemDelegate());
     QFontDatabase::WritingSystem system = delegate ? delegate->writingSystem : QFontDatabase::Any;
 
-    QFontDatabase fdb;
-    QStringList list = fdb.families(system);
+    QStringList list = QFontDatabase::families(system);
     QStringList result;
 
     int offset = 0;
     QFontInfo fi(currentFont);
 
     for (int i = 0; i < list.size(); ++i) {
-        if (fdb.isPrivateFamily(list.at(i)))
+        if (QFontDatabase::isPrivateFamily(list.at(i)))
             continue;
 
         if ((filters & scalableMask) && (filters & scalableMask) != scalableMask) {
-            if (bool(filters & QFontComboBox::ScalableFonts) != fdb.isSmoothlyScalable(list.at(i)))
+            if (bool(filters & QFontComboBox::ScalableFonts) != QFontDatabase::isSmoothlyScalable(list.at(i)))
                 continue;
         }
         if ((filters & spacingMask) && (filters & spacingMask) != spacingMask) {
-            if (bool(filters & QFontComboBox::MonospacedFonts) != fdb.isFixedPitch(list.at(i)))
+            if (bool(filters & QFontComboBox::MonospacedFonts) != QFontDatabase::isFixedPitch(list.at(i)))
                 continue;
         }
         result += list.at(i);
@@ -366,8 +367,8 @@ void QFontComboBoxPrivate::_q_updateModel()
 void QFontComboBoxPrivate::_q_currentChanged(const QString &text)
 {
     Q_Q(QFontComboBox);
-    if (currentFont.family() != text) {
-        currentFont.setFamily(text);
+    if (currentFont.families().first() != text) {
+        currentFont.setFamilies(QStringList{text});
         emit q->currentFontChanged(currentFont);
     }
 }
@@ -423,7 +424,7 @@ QFontComboBox::QFontComboBox(QWidget *parent)
         lview->setUniformItemSizes(true);
     setWritingSystem(QFontDatabase::Any);
 
-    connect(this, SIGNAL(currentIndexChanged(QString)),
+    connect(this, SIGNAL(currentTextChanged(QString)),
             this, SLOT(_q_currentChanged(QString)));
 
     connect(qApp, SIGNAL(fontDatabaseChanged()),
@@ -524,7 +525,7 @@ void QFontComboBox::setCurrentFont(const QFont &font)
 }
 
 /*!
-    \fn QFontComboBox::currentFontChanged(const QFont &font)
+    \fn void QFontComboBox::currentFontChanged(const QFont &font)
 
     This signal is emitted whenever the current font changes, with
     the new \a font.
@@ -541,7 +542,7 @@ bool QFontComboBox::event(QEvent *e)
         QListView *lview = qobject_cast<QListView*>(view());
         if (lview) {
             lview->window()->setFixedWidth(qMin(width() * 5 / 3,
-                               QDesktopWidgetPrivate::availableGeometry(lview).width()));
+                                           QWidgetPrivate::availableScreenGeometry(lview).width()));
         }
     }
     return QComboBox::event(e);
@@ -554,7 +555,7 @@ QSize QFontComboBox::sizeHint() const
 {
     QSize sz = QComboBox::sizeHint();
     QFontMetrics fm(font());
-    sz.setWidth(fm.width(QLatin1Char('m'))*14);
+    sz.setWidth(fm.horizontalAdvance(QLatin1Char('m'))*14);
     return sz;
 }
 
